mod v1;

use std::fs;
use std::io::Write;

use anyhow::Result;
pub use v1::Project;

use crate::dirs;


#[derive(Debug, Default, serde::Serialize, serde::Deserialize)]
pub struct Projects {
    list: Vec<Project>,
}

impl Projects {
    pub fn iter(&self) -> std::slice::Iter<'_, Project> {
        self.list.iter()
    }

    pub fn list(&self) -> &[Project] {
        &self.list
    }

    pub fn remove(&mut self, idx: usize) -> Project {
        self.list.remove(idx)
    }

    pub fn add(&mut self, project: Project) {
        self.list.push(project);
    }
}


impl<'a> IntoIterator for &'a Projects {
    type IntoIter = std::slice::Iter<'a, Project>;
    type Item = &'a Project;

    fn into_iter(self) -> Self::IntoIter {
        self.list.iter()
    }
}


pub fn read_projects_file() -> Result<Projects> {
    let projects_file = dirs::data_path().join("projects");

    if !projects_file.exists() {
        return Ok(Projects::default());
    }

    let file = std::fs::File::open(projects_file)?;
    let projects = bincode::deserialize_from(file)?;

    Ok(projects)
}


pub fn write_projects_file(projects: &Projects) -> Result<()> {
    let projects_file = dirs::data_path().join("projects");

    let file = fs::OpenOptions::new()
        .write(true)
        .create(true)
        .truncate(true)
        .open(projects_file)?;

    bincode::serialize_into(file, projects)?;

    Ok(())
}


pub fn clear_selected_project_file() -> Result<()> {
    let cache_file = dirs::selected_project_file();

    if fs::exists(cache_file)? {
        fs::remove_file(cache_file)?;
    }

    Ok(())
}

pub fn write_selected_project_file(project: &Project) -> Result<()> {
    let cache_file = dirs::selected_project_file();
    let path = project.path();

    let mut file = fs::OpenOptions::new()
        .write(true)
        .create(true)
        .truncate(true)
        .open(cache_file)?;
    write!(file, "{}", path.display())?;

    Ok(())
}
